import subprocess
from tempfile import TemporaryDirectory
from pathlib import Path


QISKIT_ROOT = Path("../qiskit")
RUNTIME_ROOT = Path("../qiskit-ibm-runtime")
DOCS_ROOT = Path("./").resolve()

# Expects script author to have this folder created.
MIGRATION_FOLDER = DOCS_ROOT / "migrate-open-source"
MAILMAP = MIGRATION_FOLDER / "mailmap.txt"


def main() -> None:
    with TemporaryDirectory() as tmpdir:
        qiskit_dir = clone_repo(tmpdir, repo_name="qiskit")
        git_filter_repo(
            qiskit_dir,
            "qiskit-files.txt",
            "qiskit-messages.txt",
            dest_folder="qiskit-docs",
        )
        merge_repo(qiskit_dir, remote_name="qiskit")

        runtime_dir = clone_repo(tmpdir, repo_name="qiskit-ibm-runtime")
        git_filter_repo(
            runtime_dir,
            "runtime-files.txt",
            "runtime-messages.txt",
            dest_folder="runtime-docs",
        )
        merge_repo(runtime_dir, remote_name="runtime")


def clone_repo(tmpdir: str, repo_name: str) -> Path:
    subprocess.run(
        [
            "git",
            "clone",
            "--no-tags",
            "--single-branch",
            f"https://github.com/Qiskit/{repo_name}.git",
        ],
        check=True,
        cwd=tmpdir,
    )
    return Path(tmpdir, repo_name)


def git_filter_repo(
    repo_path: str, files_txt_name: str, msg_txt_file: str, dest_folder: str
) -> None:
    subprocess.run(
        [
            "python3",
            MIGRATION_FOLDER / "git-filter-repo",
            "--paths-from-file",
            MIGRATION_FOLDER / files_txt_name,
            "--replace-message",
            MIGRATION_FOLDER / msg_txt_file,
            "--mailmap",
            MAILMAP,
            "--path-rename",
            f"docs/:{dest_folder}/",
        ],
        check=True,
        cwd=repo_path,
    )


def merge_repo(repo_path: str, remote_name: str) -> None:
    subprocess.run(["git", "remote", "add", remote_name, repo_path], check=True)
    subprocess.run(["git", "fetch", remote_name])
    subprocess.run(
        ["git", "merge", "--allow-unrelated-histories", f"{remote_name}/main"]
    )
    subprocess.run(["git", "remote", "remove", remote_name])


main()
